local codemap = require 'codemap'

-- Best Practice: use local functions
-- main() is last so it can use the local functions

MySet = codemap.set{"Fred", "Bruce", "Jim"}

local function Map()
   local A = 'Fred'
   if MySet[A] then
      trace('Hello friend')
   end   

end

SexCodeMap = codemap.map({
      M='Male',
      m='Male',
      F='Female',
      f='Female' 
   },'other')

local function MapSex()
   local V = 'M'
   local W = ''
   local A = SexCodeMap[V]
   local B = SexCodeMap[W]
   trace(A)
   trace(B)
end

RelationCodeMap = codemap.map{
   ['01']='Spouse',
   ['4f']='Grandparent',
   ['G7']='Nephew or Niece'
}

local function MapRelationship()
   local V = '4f'
   local A = RelationCodeMap[V]
   trace(A)
end

------------------------------
-- database example functions
------------------------------

-- create database, tables and data
local function CreateProcedures()
   -- create database
   local Conn = db.connect{
      api=db.SQLITE,
      name='map.sqlite',
      live = true}

   -- create table
   Conn:execute{sql=[[
      CREATE TABLE IF NOT EXISTS Procedure (
      Procedure Text(255) NOT NULL PRIMARY KEY,
      Code Text(255) )
      ]],  
      live = true}

   -- insert/replace data
   Conn:execute{sql="REPLACE INTO Procedure VALUES('procA','codeA')",  
      live = true}
   Conn:execute{sql="REPLACE INTO Procedure VALUES('procB','codeB')",  
      live = true}
   Conn:execute{sql="REPLACE INTO Procedure VALUES('procC','codeC')",  
      live = true}
end

local function ReadRset()
   local Conn = db.connect{
      api=db.SQLITE,
      name='map.sqlite',
      live = true}
   return Conn:query('SELECT * FROM Procedure')
end

local function RequestCodeMap(R)
   local t = {}
   for i = 1,#R do 
      t[R[i].Procedure:nodeValue()] = R[i].Code:nodeValue()
   end
   return codemap.map(t,'other')
end

local function MapProcedures()
   local R = ReadRset()
   local ProceduresCodeMap = RequestCodeMap(R)

   local V = R[1].Procedure:nodeValue()
   local A = ProceduresCodeMap[V]
   trace(A)

   V = 'procB'
   A = ProceduresCodeMap[V]
   trace(A)

   V = 'procD' -- any value not in the procedure table
   A = ProceduresCodeMap[V]
   trace (A)
end

function main()
   Map()
   MapSex()
   MapRelationship()

   CreateProcedures() -- create DB, table & data
   MapProcedures() 
end